/*
*
*  FileAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

//#FLAG API_V1.5_ONLY is DISABLED

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/***********************************************************************
 **
 ** Loads, updates, and deletes files and spf sets.
 **
 ***********************************************************************/
public class FileAction extends BaseAction {

  private Session session;
  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;
  private Group group;
  
  /** Static initialization routine.  Sets default values for all keywords. */
  static {
    default_keywords = new RMHashtable();
    RMProperty rp = new RMProperty("NAME","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("FILESPEC","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("REPORT-NAME","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("DESCRIPTION","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("MIME-TYPE","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("OWNER",RMLoader.username);
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("GROUP",RMLoader.group);
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("PERMISSIONS","740");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("BROWSABLE","true");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("AUTODELETE","false");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("EXPIRATION","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("KEYWORDS","");
    default_keywords.put(rp.name, rp);
    //#IF API_V1.5_ONLY DISABLED
    rp = new RMProperty("MAXVERSIONS","-1");
    default_keywords.put(rp.name, rp);
    //#END API_V1.5_ONLY
    rp = new RMProperty("NEW-CATEGORY",""); // for update action
    default_keywords.put(rp.name, rp);
  }

  /**
  *
  * Constructor method that kicks off entire process.  Clones the default
  * keywords hashtable, calls parse() and doAction().
  *
  **/
  public FileAction(Session session, Repository repository, int action, 
         StreamTokenizer st, int lineno) throws IOException{
    super(action, st, lineno);
    this.session = session;
    this.repository = repository;
    this.action = action;
    this.type = "FILE";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  /**
  *
  * After the command from the control file is parsed, doAction() initiates 
  * the execution of the command in Report Mart by determining the action
  * and calling the appropriate method.  
  *
  **/
  public int doAction() {
    switch (action) {
    case RMLoader.CREATE :
      createFile();
      break;
    case RMLoader.UPDATE :
      updateFile();
      break;
    case RMLoader.DELETE :
      deleteFile();
      break;
    default :
      error("invalid action for FILE, " + RMLoader.decodeAction(action));
    }
    return 0;
  }

  /**
  *
  * createFile() is called if the action specified in the control file
  * is "create".  createFile() checks to see if the file has already been
  * loaded, if the category specified exists, if the FILESPEC points
  * to a valid file, and determines the mime-type (SPF vs anything else)
  * and calls the appropriate loadXXX() method.
  *
  **/
  private void createFile() {
    int maxversions;

    // Separate report name from path--this assumes the last value in
    // in the path is the report name, which we might want to change
    // in the future
    Category category = null;
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String path = rmp.value;
    String full_name = path;
    String name = new String();

    for (int i = path.length() - 1; i >= 0 && path.charAt(i) != '/'; 
         name = path.charAt(i) + name, i--);
    path = path.substring(0, path.length() - name.length() - 1);

    // Check to see if the category exists in the report mart
    try {
      category = repository.getCategory(path);
    }
    catch(ReportMartException rme) {
      //check config to see what to do if category no good
      switch (RMLoader.config.when_category_missing) {
      case RMConfig.ERROR :
        error("category does not exist: " + path + ", unable to create file " +
          full_name);
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("category does not exist: " + path + ", unable to create " +
          "file " + full_name + ", writing to good file...");
        printGoodRecord();
        return;
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      }
    }
 
    // check to see that GROUP is valid
    try {
      rmp = (RMProperty)keywords.get("GROUP");
      if (!rmp.is_default)
        group = repository.getGroup(rmp.value);
    }
    catch(ReportMartException rme) {
      error("cannot find group " + rmp.value + ", " + rme.getMessage() + 
        ", cannot load file");
      printBadRecord();
      return;
    }

    // Check to see if the file preexists in the Repository
    try {
      repository.getObjectByPath(path + '/' + name);
      // if no exception occurred and we're still here, the file
      // does already exist in the Repository
      switch (RMLoader.config.when_multiple_preexists) {
        case RMConfig.ERROR :
          error("file already exists in Report Mart: " + name + ", unable to " +
            "create file");
          printBadRecord();
          return;
        case RMConfig.WARNING :
          warning("file already exists in Report Mart: " + name + ", skipping");
          printGoodRecord();
          return;
        case RMConfig.IGNORE :
          printGoodRecord();
          return;
        case RMConfig.REPLACE :
          QueryVector query_vector = null;
          Query query = null;
          // need to check here if multiple copies of the file exist
          try {
            query = repository.getQuery();
            query_vector = query.search(category,name,
                                        null,  //object type
                                        null,  //dateSince
                                        null,  //dateTo
                                        null,  //keywords
                                        false, //matchAllKeywords
                                        null,  //User
                                        false, //recursive
                                        false, //browsable
                                        true   //multiple
                                        );
            for (int i = 0; i < query_vector.size(); i++) {
              BaseObject bo = (BaseObject)query_vector.elementAt(i);
              repository.removeObjectByID(bo.getObjectID());
            }
          }
          catch (ReportMartException rme) {
            error("unable to replace file, " + rme.getMessage());
            printBadRecord();
            return;
          }
          break;
        case RMConfig.ADD :
          break;
        case RMConfig.UPDATE :
          updateFile();
          return;
        default :
          error("internal error - unexpected value found for " +
                "when_multiple_preexists: " + 
                RMLoader.config.when_multiple_preexists);
      } // switch (RMLoader.config.when_multiple_preexists) 
    }
    catch (ReportMartException rme) {
       /* file doesn't already exist, so we're okay to continue */
    }

//#IF API_V1.5_ONLY DISABLED
    // Check to see if the MAXVERSIONS argument is valid 
    try {
      rmp = (RMProperty)keywords.get("MAXVERSIONS");
      maxversions = Integer.decode(rmp.value).intValue();
    }
    catch (Exception e) {
      error("invalid MAXVERSIONS value \"" + rmp.value + "\", " + 
            e.getMessage() + ", cannot load file");
      printBadRecord();
      return;
    }
//#END API_V1.5_ONLY

    // Conditions are ok for adding file.  Check to make sure filespec is
    // given, make sure that filespec points to a valid file, then determine
    // the mime-type and load the file as either an SPFSet or a regular
    // file
    rmp = (RMProperty)keywords.get("FILESPEC");
    String filespec = rmp.value;
    if (rmp.is_default) {
      error("no input file specification given for " + name);
      printBadRecord();
      return;
    }
    File file = new File(filespec);
    if (!file.exists()) {
      switch (RMLoader.config.when_file_missing) {
      case RMConfig.ERROR :
        error("file " + filespec + " does not exist, cannot load");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("file " + filespec + " does not exist, cannot load");
        printGoodRecord();    
        return;
      case RMConfig.IGNORE :
        printGoodRecord();    
        return;
      }
    }

    // the file exists in the system.  determine the mime type and create
    // the proper object.
    ObjectType mime_type;
    rmp = (RMProperty)keywords.get("MIME-TYPE");
    if (!rmp.is_default)
      try {
        mime_type = ObjectType.getFromMimeType(session, rmp.value);
      }
      catch (ReportMartException rme) {
        error("error while trying to determine mime-type, " + 
          rme.getMessage());
        printBadRecord();
        return;
      }
    else
      mime_type = getObjectType(filespec);

    BaseObject base_object;
    if (mime_type != null && (mime_type.isSPFSet() || mime_type.isSPF())) {
       try {
         session.setActiveCategory(category);
       }
       catch (ReportMartException rme) {
         error("cannot set active category \"" + path + "\", " + 
           rme.getMessage() + ", unable to load file");
         printBadRecord();
         return;
       }
       try {
         base_object = repository.loadSPFSet(filespec);
         log("SPFSet \"" + filespec +"\" successfully loaded, " +
           "trying to set properties...");
       }
       catch (ReportMartException rme) {
         error("unable to create spf set \"" + path + '/' + name + "\"," + 
           rme.getMessage());
         printBadRecord();
         return;
       }
       try {
         category = repository.getCategory(RMLoader.config.default_category);
         session.setActiveCategory(category);
       }
       catch(ReportMartException rme) {
         warning("unable to set active category to default, " 
                 + rme.getMessage());
       }
     } // if (mime_type != null && (mime_type.isSPFSet() || mime_type.isSPF())) 
     else {
       // the mime-type was not SPFSet
       try {
         base_object = repository.loadDataObject(path, filespec);
         log("Filespec \"" + filespec +"\" successfully loaded, trying " +
           "to set properties...");
       }
       catch (ReportMartException rme) {
         error("unable to create file \"" + path + '/' + name + "\", " + 
           rme.getMessage());
         printBadRecord();
         return;
       }
    } // else (mime_type != null && (mime_type.isSPFSet() ...

    // File has been loaded into ReportMart.  Now try to set properties
    String msg = "";
    try {
      base_object.setName(name);

      rmp = (RMProperty)keywords.get("DESCRIPTION");
      msg = "error setting description, ";
      base_object.setDescription(rmp.value);

      rmp = (RMProperty)keywords.get("OWNER");
      msg = "error setting owner, ";
      base_object.setOwner(repository.getUser(rmp.value));

      rmp = (RMProperty)keywords.get("GROUP");
      msg = "error setting group, ";
      base_object.setGroup(repository.getGroup(rmp.value));

      rmp = (RMProperty)keywords.get("PERMISSIONS");
      msg = "error setting permissions, ";
      base_object.setPermissions(convertUmask(rmp.value));

      rmp = (RMProperty)keywords.get("BROWSABLE");
      msg = "error setting browsable, ";
      base_object.setAutoDelete(decodeBoolean(rmp.value));


      rmp = (RMProperty)keywords.get("EXPIRATION");
      msg = "error setting expiration, ";
      base_object.setExpiration(decodeDate(rmp.value));

      rmp = (RMProperty)keywords.get("KEYWORDS");
      msg = "error setting keywords, ";
      if (base_object instanceof SPFSet) 
        ((SPFSet)base_object).setKeywords(parseCommaSeparatedList(rmp.value));
      else 
        ((DataObject)base_object).setKeywords(
                                  parseCommaSeparatedList(rmp.value));

//#IF API_V1.5_ONLY DISABLED
      rmp = (RMProperty)keywords.get("MAXVERSIONS");
      msg = "error setting maxversions, ";
      ((VersionedObject)base_object).setMaxVersions(maxversions);
//#END API_V1.5_ONLY

      msg = "";
      base_object.update();
      log("properties successfully set");
      printGoodRecord();
    }
    catch (ReportMartException rme) {
      error(msg + rme.getMessage() + ", unable to set properties on file");
      printBadRecord();
    }

  } // private void createFile()


  /**
  *
  * Updates a given file.  If file has not been loaded, updateFile() will
  * exit, logging message, writing to good/bad record file, depending on
  * config options.
  * 
  **/
  private void updateFile(){
    SPFSet spf = null;
    DataObject file = null;
    BaseObject base = null;

    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = "";
    String path = rmp.value;
    for (int i = path.length() - 1; i >= 0 && path.charAt(i) != '/'; 
         name = path.charAt(i) + name, i--);
    path = path.substring(0, path.length() - name.length() - 1);

    QueryVector query_vector;
    try {
      Query query = repository.getQuery();
      Category category = repository.getCategory(path);
      query_vector = query.search(category,name,
                                  null,  //object type
                                  null,  //dateSince
                                  null,  //dateTo
                                  null,  //keywords
                                  false, //matchAllKeywords
                                  null,  //User
                                  false, //recursive
                                  false, //browsable
                                  true   //multiple
                                  );
    }
    catch (ReportMartException rme) {
      error("unable to update file, " + rme.getMessage());
      printBadRecord();
      return;
    }
    if (query_vector.size() == 0) {
      switch (RMLoader.config.when_file_missing) {
      case RMConfig.ERROR :
        error("file not found in repository: " + name + ", cannot update");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("file not found in repository: " + name + ", cannot update");
        printGoodRecord();
        return;
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      }
    }
    else if (query_vector.size() == 1) {
      if (updateFileProperties((BaseObject)query_vector.elementAt(0), path, 
          name)) {
        error("unable to update file");
        printBadRecord();
        return;
      }
      else {
        log("file successfully updated");
        printGoodRecord();
        return;
      }
    }
    else {
      switch (RMLoader.config.when_multiple_exists) {
        case RMConfig.ERROR :
          error("multiple copies of file exist, cannot update");
          printBadRecord();
          return;
        case RMConfig.WARNING :
          warning("multiple copies of file exist, cannot update");
        case RMConfig.IGNORE :
          printGoodRecord();
          return;
        case RMConfig.OLDEST :
          if (updateFileProperties(findOldestObject(query_vector), 
              path, name)) { 
            error("unable to update file");
            printBadRecord();
            return;
        }
        else {
          log("file successfully updated");
          printGoodRecord();
          return;
        }
      case RMConfig.NEWEST :
        if (updateFileProperties(findNewestObject(query_vector), path, name)) { 
          error("unable to update file");
          printBadRecord();
          return;
        }
        else {
          log("file successfully updated");
          printGoodRecord();
          return;
        }
      case RMConfig.ALL :
        for (int i = 0; i < query_vector.size(); i++) {
          if (updateFileProperties((BaseObject)query_vector.elementAt(i), path,
            name)) {
            error("unable to update all copies of file; " + 
              i + " copies were successfully updated");
            printBadRecord(); 
            return;
          }
        }
        log("all copies of file updated successfully");
        printGoodRecord();
      }
    }  
  } // private void updateFile()

  /**
  *
  *  Sets the properties for an update.  Reloads file if necessary.
  *  It is a separate method because it will be called from 
  *  several places within the updateFile() method.
  *
  **/
  private boolean updateFileProperties(BaseObject base_object, String path, 
        String name) {

//#IF API_V1.5_ONLY DISABLED
   // the following dummy line seems to avoid certain bugs in the ONE/API 
   // v6.0 that cause problems when attempting to update BaseObject objects
   // that are retrived from a QueryVector vector.  (These bugs appear 
   // through _at least_ v6.0.2.)
   try {
     base_object.getBrowsable();
    }
    catch (ReportMartException rme) {
      error("dummy .getBrowsable() call failed," + rme.getMessage());
      return true;
    }
//#END API_V1.5_ONLY

    boolean update_needed = false;
    String msg = "";
    RMProperty rmp; 

    if (! (base_object instanceof DataObject || 
           base_object instanceof SPFSet    )    ) {
        error("Object \"" + path + "/" + name +  "\" is neither a " +
              "DataObject nor an SPFSet, cannot update with UPDATE FILE.");
        return true;
      } // if ( ! (base_object instanceof DataObject or ... ) )
      else {
       // We have a valid type, set any properties that have been specified.
       try {
         rmp = (RMProperty)keywords.get("NEW-CATEGORY");
         if (!rmp.is_default) {
           msg = "unable to move file to new category";
           repository.getCategory(rmp.value).move(base_object);
         }

         rmp = (RMProperty)keywords.get("DESCRIPTION");
         if (!rmp.is_default) {
           msg = "unable to set description";
           update_needed = true;
           base_object.setDescription(rmp.value);
         }

         rmp = (RMProperty)keywords.get("MIME-TYPE");
         if (!rmp.is_default) {
           msg = "unable to set mime-type";
           update_needed = true;
           base_object.setObjectType(ObjectType.getFromMimeType(session, 
                                     rmp.value));
         }
   
         rmp = (RMProperty)keywords.get("OWNER");
         if (!rmp.is_default) {
           msg = "unable to set owner";
           update_needed = true;
           base_object.setOwner(repository.getUser(rmp.value));
         }
   
         rmp = (RMProperty)keywords.get("GROUP");
         if (!rmp.is_default) {
           msg = "unable to set group";
           update_needed = true;
           base_object.setGroup(repository.getGroup(rmp.value));
         }
   
         rmp = (RMProperty)keywords.get("PERMISSIONS");
         if (!rmp.is_default) {
           msg = "unable to set permissions";
           update_needed = true;
           base_object.setPermissions(convertUmask(rmp.value));
         }
   
         rmp = (RMProperty)keywords.get("BROWSABLE");
         if (!rmp.is_default) {
           msg = "unable to set browsable";
           update_needed = true;
           base_object.setBrowsable(decodeBoolean(rmp.value));
         }

         rmp = (RMProperty)keywords.get("AUTODELETE");
         if (!rmp.is_default) {
           msg = "unable to set autodelete";
           update_needed = true;
           base_object.setAutoDelete(decodeBoolean(rmp.value));
         }

         rmp = (RMProperty)keywords.get("EXPIRATION");
         if (!rmp.is_default) {
           msg = "unable to set expiration";
           Date exp_date = decodeDate(rmp.value);
           update_needed = true;
           base_object.setExpiration(exp_date);
         }

         rmp = (RMProperty)keywords.get("KEYWORDS");
         if (!rmp.is_default) {
           msg = "unable to set keywords";
           update_needed = true;
           if (base_object instanceof SPFSet) 
             ((SPFSet)base_object).setKeywords(parseCommaSeparatedList(
               rmp.value));
           else 
             ((DataObject)base_object).setKeywords(parseCommaSeparatedList(
               rmp.value));
          }
   
//#IF API_V1.5_ONLY DISABLED
         rmp = (RMProperty)keywords.get("MAXVERSIONS");
         if (!rmp.is_default) {
            try {
            // debugging lines here because of maxversion problems 
            // in API for 6.0.1
            msg = "error getting Maxversions";
            System.out.println("Prior Maxversions: " + 
                     ((VersionedObject)base_object).getMaxVersions() );
            }
            catch (Exception e) { 
            } 
 
          msg = "error setting MAXVERSIONS ";

          int maxversions;
          // Check to see if the MAXVERSIONS argument is valid 
          try {
            maxversions = Integer.decode(rmp.value).intValue();
          }
          catch (Exception e) {
            error("invalid MAXVERSIONS value \"" + rmp.value + "\", " 
                   + e.getMessage() );
            printBadRecord();
            return true;
          }
 
          update_needed = true;
          ((VersionedObject)base_object).setMaxVersions(maxversions);
         } // if (!rmp.is_default)  -- MAX-VERSIONS
//#END API_V1.5_ONLY
    
       msg = "error during .update() call" ;
       if ( update_needed ) base_object.update();
          
       } //  try 
       catch (ReportMartException rme) {
         error(msg + ": " + rme.getMessage() + "; unable to update file");
          rme.printStackTrace();
         return true;
       }
    } // if ( ! (base_object instanceof DataObject or ... ) ) ... else

    rmp = (RMProperty)keywords.get("FILESPEC");
    if (!rmp.is_default) {
      // need to reload the file
   
      String filespec = rmp.value;
      File file = new File(filespec);
      if (!file.exists()) {
        switch (RMLoader.config.when_file_missing) {
          case RMConfig.ERROR :
            error("file does not exist, " + rmp.value + ", cannot load");
            return true;
          case RMConfig.WARNING :
            warning("file does not exist, " + rmp.value + ", cannot load");
            return false;
          case RMConfig.IGNORE :
            return false;
        } // switch (RMLoader.config.when_file_missing) 
      } // if (!file.exists())



      if (base_object instanceof DataObject) {
//#IF API_V1.5_ONLY DISABLED
        switch (RMLoader.config.when_reloading_file) {
          case RMConfig.NEWVERSION :
            try {
      
              int max_versions, version_count;
      
              max_versions = ((VersionedObject)base_object).getMaxVersions();
              version_count = ((VersionedObject)base_object).getVersionCount();
        
              if ( max_versions > -1 && version_count >= max_versions ) {
                 switch (RMLoader.config.when_maxversions_reached) {
                   case RMConfig.ERROR :
                     error("version limit reached, unable to load new " +
                           "version from " + filespec + ".");
                     return true;
                   case RMConfig.WARNING :
                     warning("version limit reached, unable to load new " +
                             "version from " + filespec + ".");
                     return false;
                   case RMConfig.IGNORE :
                     printGoodRecord();
                     return false;
                   case RMConfig.PURGE :
             
              
                     // Purge the oldest versions until we've purged enough
                     // to let us add our new version below.
                     // (Generations are numbered "0" to "(version_count - 1)".)
                     for (int i = version_count - 1;
                          i >= max_versions - 1;  
                          i-- )  {
       
                        ((VersionedObject)base_object).deleteVersion(i);
      
                     } 
                   } // switch (RMLoader.config.when_multiple_preexists) 
      
              } // if ( max_versions > -1 && version_count >= max_versions ) 
      
              ((VersionedObject)base_object).addVersion(filespec);

              // update() not needed for addVersion, apparently
              log("\"" + filespec +"\" successfully loaded as a new version" );
            }
            catch (ReportMartException rme) {
               error ("unable to load new version from \"" + filespec + "\", " +
                       rme.getMessage());
               return true;
            }
            break;
          case RMConfig.REPLACE :
            try {
              ((VersionedObject)base_object).replaceContentsFromFile(filespec);
              base_object.update();
              log("\"" + filespec + 
                  "\" successfully replace existing contents" );
            }
            catch (ReportMartException rme) {
              error("unable to replace version from \"" + filespec + "\", " +
                rme.getMessage());
                return true;
            }
            break;
          default:
            error("Internal error, unrecognized " + 
                  "RMLoader.config.when_reloading_file value \"" + 
                  RMLoader.config.when_reloading_file + "\""); 
            return true; 
        } // switch (RMLoader.config.when_reloading_file)
//#END API_V1.5_ONLY
//#IF API_V1.5_ONLY ENABLED
//!        try {
//!          base_object = repository.replaceDataObject(path+name, rmp.value);
//!        }
//!        catch (ReportMartException rme) {
//!          error("unable to load file \"" + rmp.value + "\", " +
//!            rme.getMessage());
//!            return true;
//!        }
//#END API_V1.5_ONLY

      } // if (base_object instanceof DataObject) 
      else if (base_object instanceof SPFSet) {
        try {
          // note: this currenly looses all attributes of the SPFSet 
          // (description, etc.) when we delete the orginal file.
          // we may want to preserve them before the delete and
          // reset them after the new one is created.  Or maybe we 
          // shouldn't allow UPDATEs to SPFSets with FILESPECs
          // specified?  (How does the admin tool do "versions"
          // for SPFSets?

          repository.removeObjectByID(base_object.getObjectID());
          session.setActiveCategory(repository.getCategory(path));
          base_object = repository.loadSPFSet(rmp.value);
          session.setActiveCategory(repository.getCategory(
            RMLoader.config.default_category));
        }
        catch (ReportMartException rme) {
          error("unable to load file \"" + rmp.value + "\", " + 
            rme.getMessage());
          return true;
        }
      } // else if (base_object instanceof SPFSet) 
    } //  if (!rmp.is_default) 

    return false;
  } // boolean updateFile(...)


  /**
  *
  * Delete the specified file.  deleteFile() checks to make sure the file
  * exists, that only one exists, and that all keyword values are default.
  * Generates errors or warnings (depending on config options).
  *
  **/
  private void deleteFile(){
    boolean error_status = false;
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    String report_name = new String();
    String path = name;
    for (int i = path.length() - 1; i >= 0 && path.charAt(i) != '/'; 
         report_name = path.charAt(i) + report_name, i--);
    path = path.substring(0, path.length() - report_name.length() - 1);

    // check for ignored keywords
    if (anyKeywordIgnored()) {
      printBadRecord();
      return;
    }

    QueryVector query_vector = null;
    Category category = null;
    Query query = null;
    // need to check here if multiple copies of the file exist
    try {
      query = repository.getQuery();
      category = repository.getCategory(path);
      query_vector = query.search(category,report_name,
                                  null,  //object type
                                  null,  //dateSince
                                  null,  //dateTo
                                  null,  //keywords
                                  false, //matchAllKeywords
                                  null,  //User
                                  false, //recursive
                                  false, //browsable
                                  true   //multiple
                                  );
    }
    catch (ReportMartException rme) {
      error("unable to run query, " + rme.getMessage() + ", unable to delete " +
        "file " + name);
      printBadRecord();
      return;
    }

    if (query_vector.size() > 1) {
      switch (RMLoader.config.when_multiple_exists) {
      case RMConfig.ERROR :
        error("multiple copies of file " +  name + " exist, " + 
          "cannot delete");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("multiple copies of file " + name + " exist, cannot delete");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      case RMConfig.OLDEST :
        try {
          repository.removeObjectByID(
              findOldestObject(query_vector).getObjectID());
          log("File \"" + name + "\" successfully deleted");
          printGoodRecord();
        }
        catch (ReportMartException rme) { 
          error("uanble to delete file, " + rme.getMessage());
          printBadRecord();
          return; 
        }
        break;
      case RMConfig.NEWEST :
        try {
          repository.removeObjectByID(
              findNewestObject(query_vector).getObjectID());
          log("File \"" + name + "\" successfully deleted");
          printGoodRecord();
        }
        catch (ReportMartException rme) { 
          error("uanble to delete file, " + rme.getMessage());
          printBadRecord();
          return; 
        }
        break;
      case RMConfig.ALL :
        try {
          for (int i = 0; i < query_vector.size(); i++) {
            BaseObject base = (BaseObject)query_vector.elementAt(i);
            repository.removeObjectByID(base.getObjectID());
          }
          log("File \"" + name + "\" successfully deleted");
          printGoodRecord();
        }
        catch (ReportMartException rme) {
          error("unable to delete file \"" + name + "\", " +
            rme.getMessage());
          printBadRecord();
        }
      }
    }
    else {
      try {
        repository.removeObjectByPath(rmp.value);
        printGoodRecord();
      }
      catch (ReportMartException rme) {
        error("Error deleting file \"" + rmp.value + "\", " +
                        rme.getMessage());
        printBadRecord();
      }
    }
  }

  /**
  *
  * Returns an ObjectType based on the file name.
  *
  **/
  private ObjectType getObjectType(String filespec) {
    try {
      int len = filespec.length() - 1;
      String s = new String();
      char ch = '-';
      while (ch != '.' && len >= 0) {
        ch = filespec.charAt(len);
        if (ch != '.')
          s = ch + s;
        len--;
      }
      ObjectType ot[] = ObjectType.getFromExtension(session, s);
      if (ot == null) {
        warning(filespec + " has an unrecognized extension " + s);
        return null;
      }
      else {
        return ot[0];
      }
    }
    catch (ReportMartException rme) {
      error("Error determining mime-type from file \"" + filespec +
                      "\"");
      return null;
    }
  } // private ObjectType
}
